require "fox"
require "responder"

include Fox

class NonModalDialogBox < FXDialogBox

  def initialize(owner)
    # Invoke base class initialize function first
    super(owner, "Test of Dialog Box", 
          DECOR_TITLE|DECOR_BORDER)

    text_options = JUSTIFY_RIGHT | FRAME_SUNKEN | 
                   FRAME_THICK | LAYOUT_SIDE_TOP
    @text_field = FXTextField.new(self, 20, nil, 0, 
                  text_options)
    @text_field.setText("")

    layout_options = LAYOUT_SIDE_TOP | FRAME_NONE | 
                     LAYOUT_FILL_X | LAYOUT_FILL_Y | 
                     PACK_UNIFORM_WIDTH
    layout = FXHorizontalFrame.new(self, layout_options)

    options = FRAME_RAISED | FRAME_THICK | 
              LAYOUT_RIGHT | LAYOUT_CENTER_Y
    FXButton.new(layout, "&Hide", nil, self, ID_HIDE, 
                 options)
  end

  def onCmdCancel
    @text_field.setText("")
    super
  end

  def getText
    @text_field.getText
  end
end

class ModalDialogBox < FXDialogBox

  def initialize(owner)
    # Invoke base class initialize function first
    super(owner, "Test of Dialog Box", 
          DECOR_TITLE|DECOR_BORDER)

    text_options = JUSTIFY_RIGHT | FRAME_SUNKEN | 
                   FRAME_THICK | LAYOUT_SIDE_TOP
    @text_field = FXTextField.new(self, 20, nil, 0, 
                  text_options)
    @text_field.setText("")

    layout_options = LAYOUT_SIDE_TOP | FRAME_NONE | 
                     LAYOUT_FILL_X | LAYOUT_FILL_Y | 
                     PACK_UNIFORM_WIDTH
    layout = FXHorizontalFrame.new(self, layout_options)

    options = FRAME_RAISED | FRAME_THICK | 
              LAYOUT_RIGHT | LAYOUT_CENTER_Y
    FXButton.new(layout, "&Cancel", nil, self, 
                 ID_CANCEL, options)
    FXButton.new(layout, "&Accept", nil, self, 
                 ID_ACCEPT, options)
  end

  def onCmdCancel
    @text_field.setText("")
    super
  end

  def getText
    @text_field.getText
  end
end

class DialogTestWindow < FXMainWindow

  include Responder

  # Message identifiers
  ID_NON_MODAL, 
  ID_MODAL, 
  ID_TIMER = enum(FXMainWindow::ID_LAST, 3)

  def initialize(app)
    # Invoke base class initialize first
    super(app, "Dialog Test", nil, nil, 
          DECOR_ALL, 0, 0, 400, 200)

    # Set up the message map for this window
    FXMAPFUNC(SEL_COMMAND, ID_NON_MODAL, 
              "onCmdNonModelDialog")
    FXMAPFUNC(SEL_COMMAND, ID_MODAL,     
              "onCmdModalDialog")
    FXMAPFUNC(SEL_TIMEOUT, ID_TIMER,     
              "onCmdTimer")

    layout_options = LAYOUT_SIDE_TOP | FRAME_NONE | 
                     LAYOUT_FILL_X | LAYOUT_FILL_Y | 
                     PACK_UNIFORM_WIDTH
    layout = FXHorizontalFrame.new(self, layout_options)

    button_options = FRAME_RAISED | FRAME_THICK | 
                     LAYOUT_CENTER_X | LAYOUT_CENTER_Y
    FXButton.new(layout, "&Non-Modal Dialog...", nil, 
                 self, ID_NON_MODAL, button_options)
    FXButton.new(layout, "&Modal Dialog...",     nil, 
                 self, ID_MODAL, button_options)
  
    @timer = getApp().addTimeout(1000, self, ID_TIMER);
    @non_modal_dialog = NonModalDialogBox.new(self)
  end

  def onCmdNonModelDialog(sender, sel, ptr)
    @non_modal_dialog.show(PLACEMENT_OWNER)
  end

  def onCmdModalDialog(sender, sel, ptr)
    dialog = ModalDialogBox.new(self)
    if dialog.execute(PLACEMENT_OWNER) == 1
      puts dialog.getText
    end
    return 1
  end

  def onCmdTimer(sender, sel, ptr)
    text = @non_modal_dialog.getText
    unless text == @previous
      @previous = text
      puts @previous
    end
    @timer = getApp().addTimeout(1000, self, 
                                 ID_TIMER);
  end

  def create
    super
    show(PLACEMENT_SCREEN)
  end
end

def run
  application = FXApp.new("DialogTest", 
                          "Sample programs")
  application.init(ARGV)
  DialogTestWindow.new(application)
  application.create
  application.run
end

run
