require "socket"
require "timeout"
    
ChessServer     = '127.0.0.1'  # Replace this IP address
ChessServerPort = 12000
PeerPort        = 12001

WHITE, BLACK = 0, 1
Colors = %w[White Black]

def drawBoard(board)
  puts <<-EOF
+----------------------------------+
| Stub! Drawing the board here...  |
+----------------------------------+
  EOF
end

def analyzeMove(who, move, num, board)
  # Stub - black always wins on 4th move
  if who == BLACK and num == 4
    move << "  Checkmate!"
  end
  true  # Stub again - always say it's legal.
end

def ended?(myColor, whoseMove, 
           whiteName, blackName, move)
  opponent = (myColor==WHITE ? blackName : whiteName)
  case move
    when /resign/i     # Player explicitly resigns
      if myColor == whoseMove
        puts "You've resigned. #{opponent} wins."
      else
        puts "#{opponent} resigns. You win!"
      end
      return true
    when /Checkmate/   # analyzeMove detects mate
      if myColor == whoseMove
        puts "You have checkmated #{opponent}."
      else
        puts "#{opponent} has checkmated you."
      end
      return true
  end
  return false
end

def myMove(who, lastmove, num, board, sock)
  # lastmove not actually used in dummy example
  ok = false
  until ok do
    print "\nYour move: "
    move = STDIN.gets.chomp
    ok = analyzeMove(who, move, num, board)
    puts "Illegal move" if not ok
  end
  sock.puts move
  drawBoard(board)
  move
end

def otherMove(who, lastmove, num, board, sock)
  # lastmove not actually used in dummy example
  move = sock.gets.chomp
  puts "\nOpponent: #{move}"
  drawBoard(board)
  move
end

def setupWhite(opponent)
  puts "\nWaiting for a connection..."
  server = TCPServer.new(PeerPort)
  session = server.accept
  str = nil
  begin
    timeout(30) do
      str = session.gets.chomp
      if str != "ready"
        raise "Protocol error: ready-message was #{str}"
      end
    end
  rescue TimeoutError
    raise "Did not get ready-message from opponent."
  end
  puts "Playing #{opponent}... you are white.\n"
  session              # Return connection
end

def setupBlack(opponent, ipname)
  # Asymmetrical because we assume black is the client
  puts "\nConnecting..."
  socket = TCPSocket.new(ipname, PeerPort)
  socket.puts "ready"
  puts "Playing #{opponent}... you are black.\n"
  socket               # Return connection
end


# "Main"...

if ARGV.size != 2
  puts "Parameters required: myname opponent[:hostname]"
  exit
end

myself, opponentID = ARGV
opponent = opponentID.split(":")[0]   # Remove hostname

# Contact the chess server

print "Connecting to the chess server... "
STDOUT.flush

socket = TCPSocket.new(ChessServer, ChessServerPort)

socket.puts "login #{myself} #{opponentID}"
response = socket.gets.chomp

puts "got response.\n"

name, ipname, oppColor = response.split ":"
oppColor = oppColor.to_i
myColor = if oppColor == WHITE then BLACK else WHITE end

move = nil
board = nil      # Not really used in this dummy example
num = 0

if myColor == WHITE         # We're white
  who, opp = WHITE, BLACK
  session = setupWhite(opponent)
  drawBoard(board)
  loop do
    num += 1
    move = myMove(who, move, num, board, session)
    break if ended?(who, who, myself, opponent, move)
    move = otherMove(who, move, num, board, session)
    break if ended?(who, opp, myself, opponent, move)
  end
else                        # We're black
  who, opp = BLACK, WHITE
  socket = setupBlack(opponent, ipname)
  drawBoard(board)
  loop do
    num += 1
    move = otherMove(who, move, num, board, socket)
    break if ended?(who, opp, opponent, myself, move)
    move = myMove(who, move, num, board, socket)
    break if ended?(who, who, opponent, myself, move)
  end
  socket.close 
end
